 function [cox, six, eix]=expi(x,s);  % 010506, 010516, 011029,011201,011205
% Computation of cos(x), sin(x), exp(ix) in mulprec,  x real.
%  A new version of expi.m that does not use the global constants pilo4
% and expi4; it can thus be used if more than 50 gytes are demanded.
% (Not quite true; if |x|>15*pi the accuracy of pilong restricts the
% the number of gytes, see line 21.)
% The same program without comments is found in expi.m .
% The old version is now called expiold.m. It is not yet clear whether
% the new version is preferable in all respects. If |x|<pi/8, the new
% version is undoubtedly better.

global P logP log2 pilong
%  P=10^7; logP=log(P); log2=log(2);
if length(x)==1, xflo=x; x=npr(x);
else             xflo=flo(x);
end
n=floor((xflo+pi)/pi/2);
if abs(n)>8, xbar=rnize(sub(x, mul(n,chop(pilong,s+1))));
   % See comment above. "8" can replaced.
else     xbar=x;
end
%
%    Set xm=i*xbar/2^m. e^xm will be represented by the first k terms
%  of its Maclaurin expansion. Finally e^(ix)=e^(i*xbar) is obtained
%  by squaring e^xm  m times. It is assumed that the computational
%  work is approx. prop. to m+c*k, where c is determined by rough
%  analysis.
%    The integers k and m are determined by minimizing (approx.)
%  m+c*k, subject to the constraint of setting the bounds for the
%  Maclaurin trunc. err. and for the chopping err. equal, i.e.,roughly
%  |xm|^k/k! =P^(-s). This yields (approx.) k=0.7/c + 4.8*sqrt(s/c);
%  m=c*k+log_2|xbar/k>; these formulas are used in the program.
%  Due to the squarings this error bound is to be multiplied by
%  2^m; it typically helps to increase s by 1 (if s<=50)!
c=0.4;  k=0.7/c+4.8*sqrt((s+1)/c);
% k=1.5*(1/log2+sqrt(log2^(-2) + logP*s/1.5/log2 ));
m= max(0, ceil(c*k + log(abs(flo(xbar)/k))/log2)),                %%
k=ceil(k),                                                        %%
m2=npr(2^m);  tmp=div(xbar, m2, s+1);
xm=[tmp(1), i*tmp(2:length(tmp))];
% Note that xm is imaginary, while x and xbar are real.
%
%   Now comes a "Horner" evaluation of the truncated Taylor
% expansion for alp=e^(xm). All coefficients (bet) are multiplied by
% exact values of (k-1)! .
% for
bet=1; alp=bet;
for j=k-1:-1:1,
   sj=ceil(s+1-(s-3)*(j-1)/(k-2));
   bet=mul(j,bet);
   tmp=mul(xm,alp,sj);
   alp= chop( add( tmp, bet), sj);      % The last sj equals s+1.
end
alp=div(alp,bet,s+1); bet=rnize(bet),                            %%

% alp had to be divided by (k-1)!, in order to become e^(xm). By this
% gyte strategy the last value of  bet is exactly equal to this factorial.
%
% Next, square alp m times in order to obtain alp=(e^xm)^(2^m)=e^(i*xbar):

for j=1:m,
   tmp=mul(alp,alp,s+1);
   alp=tmp;
end

% Now alp=e^(i*xbar)=e^(i*x), normalized to s+1 gytes.

eix=chop(alp,s); cox=real(eix); six=imag(eix); six(1)=eix(1);

%   THIS IS THE END of  expi.m .
