%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  23. lntemp.m  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y=lntemp(x,s); % x>0.  % 011025,
% too a large extent coinciding with lnr.m.
% Natural logarithm in mulprec chopped to s gytes. (Domain of x?)
% Initial approximation is the Matlab value of log(x) in normal
% precision.Improved values are obtained by Newton's method
% for the equation e^y=x.
% Uses the function extemp.m .
% To be used if one needs to compute a logarithm to a larger number
% of gytes than LP has at present. In particular, it is to be used to provide
% a more accurate value of L10 (and hence LP).
%
if length(x)>1, x=nize(x); xflo=flo(x);
else            xflo=x;    x=npr(xflo);
end
%
% Initial approximation:
if xflo<=0, y=NaN; return,
% make also an error message if x is non-real.
else
   yp=npr(log(xflo));
   alp=yp; bet=x;
%
   iter=ceil(log(s-2)/log(2)); one=npr(1);
%
% Newton iterations:
   for j=1:iter,
      tmp=extemp(mi(alp),s);   % tmp=exp(-alp)
      bet=mul(bet,tmp,s+1);
      alp= sub(bet,one);
      % alp=bet-1 approx -alp^2/2 except for the first time
      yp = add(yp,alp);      % yp:=yp+alp
   end
   y=nize(yp);
end

