function z =recip(x,s);      % 01-05-14, comments revised 01-11-12.
% Returns the reciprocal, z=1/x chopped to s gytes.
% The parameter s is optional. x and z may be complex.
% z is obtained by Newton's quadratic convergent iteration method for
% the equation 1/z - x = 0; x and z may be complex.
%
if length(x)==1, zp=npc(1/x); x=npc(x);        % if x is a float,
else zp=npc(1/flo(x));                         % if x is a mulprec number.
  % 1st approx. obtained by a float division to almost 16 decimal places.
end
%
if nargin==1, s=max(6,length(x));   end        % default value of s.
iter=floor(log(s-1)/log(2));
% # iterations, estimated by a rather simple anlysis of the convergence.
s1=5;
while s1< max(s+1,6)
  % Newton improvements for the eqn. 1/z-x=0:
  s1=min(s+1,2*s1-2)
  slask=mul(x,zp,s1);
  zp=mul( sub([0 2],slask),zp, s1);
% i.e., z_{i+1}=z_i*(2-x*z_i), the result is chopped to suitable length.
end

%for i=1:iter,    % older version
%  % Newton improvements for the eqn. 1/z-x=0:
%  slask=mul(x,zp,s+1);
%  zp=mul( sub([0 2],slask),zp, s+1);
% i.e., z_{i+1}=z_i*(2-x*z_i), the result is chopped to suitable length.
%end
%z=zp;
z=chop(zp,s); % Why? z was normalized in the last multiplication.
